/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.common.portability.internal.linux;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.StringReader;
import java.util.Arrays;
import java.util.HashSet;

import org.junit.Test;

public class LinuxProcfsDataExtractorTest {

    @Test
    public void threeLibs() {
        HashSet<String> expectedLibs = new HashSet<>(Arrays.asList("/usr/foo/libhello.so",
                "/usr/bar/libworld.so.0.18.0", "/tmp/libnew.so.so"));
        StringReader threeLibs = new StringReader(
                "000000000000-00000000f000 r-xp 00000000 fd:01 1234567                    /usr/foo/libhello.so\n" +
                "00000000f000-000000050000 ---p 00085000 fd:02 1234567                    /usr/bar/libworld.so.0.18.0\n" +
                "000000050000-000000e00000 r--p 00085000 fd:03 1234567                    /tmp/libnew.so.so");
        // Cannot compare arrays directly since the implementation internally gathers libs
        // to a Set, which might yield a different ordering of items in the collection in the end
        assertEquals(expectedLibs, new HashSet<>(Arrays.asList(LinuxProcfsDataExtractor.getNativeLibs(threeLibs))));
    }

    @Test
    public void noLibs() {
        StringReader noLibs = new StringReader(
                "000000000000-00000000f000 r-xp 00000000 fd:01 1234567                    /usr/foo/libhello\n" +
                "00000000f000-000000050000 ---p 00085000 fd:02 1234567                    /usr/bar/libworld\n" +
                "000000050000-000000e00000 r--p 00085000 fd:03 1234567                    /tmp/libnew");
        String[] result = LinuxProcfsDataExtractor.getNativeLibs(noLibs);
        int numExpectedLibs = 0;

        assertNotNull(result);
        assertEquals(result.length, numExpectedLibs);

    }

    @Test
    public void emptyFileNativeLibs() {
        StringReader empty = new StringReader("");
        String[] result = LinuxProcfsDataExtractor.getNativeLibs(empty);
        int numExpectedLibs = 0;

        assertNotNull(result);
        assertEquals(result.length, numExpectedLibs);

    }

    @Test
    public void emptyFileResidentMemory() {
        StringReader empty = new StringReader("");
        Long result = LinuxProcfsDataExtractor.getResidentMemorySize(empty);

        assertNull(result);
    }

    @Test
    public void missingResidentMemoryStat() {
        StringReader resMemMissing = new StringReader(
            "Uid:    1000    1000    1000    1000\n" +
            "Gid:    100     100     100     100\n" +
            "FDSize: 256\n" +
            "Groups: 16 33 100\n" +
            "NStgid: 17248\n" +
            "NSpid:  17248\n" +
            "NSpgid: 17248\n" +
            "NSsid:  17200\n" +
            "VmPeak:     131168 kB\n" +
            "VmSize:     131168 kB\n" +
            "VmLck:           0 kB\n" +
            "VmPin:           0 kB\n" +
            "VmHWM:       13484 kB\n" +
            "RssAnon:     10264 kB\n" +
            "RssFile:      3220 kB\n" +
            "RssShmem:        0 kB\n" +
            "VmData:      10332 kB\n" +
            "VmStk:         136 kB\n" +
            "VmExe:         992 kB\n" +
            "VmLib:        2104 kB\n" +
            "VmPTE:          76 kB\n" +
            "VmPMD:          12 kB\n" +
            "VmSwap:          0 kB");
        Long result = LinuxProcfsDataExtractor.getResidentMemorySize(resMemMissing);

        assertNull(result);
    }

    @Test
    public void residentMemoryStatPresent() {
        StringReader resMemPresent = new StringReader(
            "Uid:    1000    1000    1000    1000\n" +
            "Gid:    100     100     100     100\n" +
            "FDSize: 256\n" +
            "Groups: 16 33 100\n" +
            "NStgid: 17248\n" +
            "NSpid:  17248\n" +
            "NSpgid: 17248\n" +
            "NSsid:  17200\n" +
            "VmPeak:     131168 kB\n" +
            "VmSize:     131168 kB\n" +
            "VmLck:           0 kB\n" +
            "VmPin:           0 kB\n" +
            "VmHWM:       13484 kB\n" +
            "VmRSS:       13484 kB\n" +
            "RssAnon:     10264 kB\n" +
            "RssFile:      3220 kB\n" +
            "RssShmem:        0 kB\n" +
            "VmData:      10332 kB\n" +
            "VmStk:         136 kB\n" +
            "VmExe:         992 kB\n" +
            "VmLib:        2104 kB\n" +
            "VmPTE:          76 kB\n" +
            "VmPMD:          12 kB\n" +
            "VmSwap:          0 kB");
        Long result = LinuxProcfsDataExtractor.getResidentMemorySize(resMemPresent);

        long expected = 13484;

        assertEquals(result.longValue(), expected);
    }

}
