/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.jvm.overview.agent.internal.model;
;
import java.net.URI;
import java.util.Arrays;
import java.util.Collections;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.redhat.thermostat.lang.schema.models.Timestamp;
import com.sun.org.apache.bcel.internal.generic.RETURN;
import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.ConfigurationPolicy;
import org.apache.felix.scr.annotations.Modified;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.agent.http.RequestFailedException;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.BackendConfigurationUtil;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.lang.schema.JSONService;
import com.redhat.thermostat.lang.schema.annotations.Schema;
import com.redhat.thermostat.jvm.overview.agent.model.VmId;
import com.redhat.thermostat.jvm.overview.agent.model.VmInfo;
import com.redhat.thermostat.storage.core.AgentId;

@Component(
        policy = ConfigurationPolicy.REQUIRE,
        configurationPid = "jvm-overview"
)
@Service(VmInfoDAO.class)
public class VmInfoDAOImpl implements VmInfoDAO {

    private static final String PLUGIN_ID = "jvm-overview";
    private static final String SYSTEM_PATH = "systems/";
    private static final String VM_PATH = "/jvms/";

    private final Logger logger = LoggingUtils.getLogger(VmInfoDAOImpl.class);

    private final ConfigurationCreator configCreator;

    private URI gatewayURL;

    @Reference
    private ConfigurationInfoSource configInfoSource;

    @Reference
    private HttpRequestService httpRequestService;

    @Reference
    private JSONService jsonService;

    @Reference
    private SystemID systemID;

    public VmInfoDAOImpl() throws Exception {
        this(new ConfigurationCreator(), null);
    }

    VmInfoDAOImpl(ConfigurationCreator creator, ConfigurationInfoSource source) throws Exception {
        this.configCreator = creator;
        this.configInfoSource = source;
    }

    @Activate
    public void activate() throws Exception {
        BackendConfigurationUtil config = configCreator.create(configInfoSource);
        this.gatewayURL = config.getGatewayURL();
    }

    /*
     * Default OSGI behaviour is to restart the component when configuration changes.
     * The @modified method acts as a callback to prevent this from happening since
     * we don't need to restart the component for our use-case.
     */
    @Modified
    protected void modified() {
        logger.config("Doing nothing on configuration change for plugin jvm-overview");
    }

    @Override
    public VmInfo getVmInfo(final VmId id) {
        return null; // TODO Remove once VM Id completer is removed
    }

    @Override
    public Set<VmId> getVmIds(AgentId agentId) {
        return Collections.emptySet(); // TODO Remove once VM Id completer is removed
    }

    @Override
    public void putVmInfo(final VmInfo info) {
        try {
            // Encode as JSON and send as POST request
            String json = jsonService.serialiase(Arrays.asList(info));
            URI uri = getAddURI();
            httpRequestService.sendHttpRequest(json, uri, HttpRequestService.Method.POST);
        } catch (RequestFailedException e) {
            logger.log(Level.WARNING, "Failed to send JVM information to web gateway");
            logger.log(Level.INFO, e.getMessage());
        }
    }

    @Override
    public void putVmStoppedTime(final String agentId, final String vmId, final long timestamp) {
        URI uri = getUpdateURI(vmId);
        try {
            // Encode as JSON and send as PUT request
            VmInfoTimestampUpdate update = new VmInfoTimestampUpdate(timestamp);
            String json = jsonService.serialiase(update);
            httpRequestService.sendHttpRequest(json, uri, HttpRequestService.Method.PUT);
        } catch (RequestFailedException e) {
            logger.log(Level.WARNING, "Failed to send JVM stop update to web gateway at: " + uri);
            logger.log(Level.INFO, e.getMessage());
        }
    }

    @Override
    public void updateVmNativeLibs(String vmId, String[] newLibs) {
        URI uri = getUpdateURI(vmId);
        try {
            // Encode as JSON and send as PUT request
            VmInfoNativeLibUpdate update = new VmInfoNativeLibUpdate(newLibs);
            String json = jsonService.serialiase(update);
            httpRequestService.sendHttpRequest(json, uri, HttpRequestService.Method.PUT);
        } catch (RequestFailedException e) {
            logger.log(Level.WARNING, "Failed to send JVM native library update to web gateway at: " + uri);
            logger.log(Level.INFO, e.getMessage());
        }
    }

    private URI getAddURI() {
        StringBuilder builder = new StringBuilder();
        builder.append(SYSTEM_PATH);
        builder.append(systemID.getSystemID());
        return gatewayURL.resolve(builder.toString());
    }

    private URI getUpdateURI(String vmId) {
        StringBuilder builder = new StringBuilder();
        builder.append(SYSTEM_PATH);
        builder.append(systemID.getSystemID());
        builder.append(VM_PATH);
        builder.append(vmId);
        return gatewayURL.resolve(builder.toString());
    }

    protected void bindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = httpRequestService;
    }

    protected void unbindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = null;
        logger.log(Level.INFO, "Unbound HTTP service. Further attempts to store data will fail until bound again.");
    }

    protected void bindJsonService(JSONService jsonService) {
        this.jsonService = jsonService;
    }

    protected void bindSystemId(SystemID id) {
        this.systemID = id;
    }

    protected void unbindSystemId(SystemID id) {
        this.systemID = null;
    }


    static class VmInfoTimestampUpdate {

        static class UpdateSet {

            @Schema
            long stopTime;

            UpdateSet(long ts) {
                stopTime = ts;
            }
        }

        @Schema
        UpdateSet set;

        VmInfoTimestampUpdate(long ts) {
            this.set = new UpdateSet(ts);
        }

        long getStopTime() {
            return set.stopTime;
        }

    }

    static class VmInfoNativeLibUpdate {

        static class UpdateSet {

            @Schema
            private String[] loadedNativeLibraries;

            UpdateSet(String[] param) {
                loadedNativeLibraries = param;
            }
        }

        @Schema
        UpdateSet set;

        VmInfoNativeLibUpdate(String[] param) {
            set = new UpdateSet(param);
        }

        String[] getLoadedNativeLibraries() {
            return set.loadedNativeLibraries;
        }
    }

    // For Testing purposes
    static class ConfigurationCreator {

        BackendConfigurationUtil create(ConfigurationInfoSource source) {
            return new BackendConfigurationUtil(source, PLUGIN_ID);
        }

    }
}
